{$DENYPACKAGEUNIT}

unit IndyApp;

interface

uses
  Classes, WebBroker, IndyHTTP, IdHTTPServer, IdTCPServer;

type
  TIndyFactory = class;

  TIndyApplication = class(TWebApplication)
  private
    FServer: TIdHTTPServer;
    FFactory: TIndyFactory;
    procedure SetFactory(Value: TIndyFactory);
    function GetFactory: TIndyFactory;
    function NewRequest(Request: TIdHTTPRequestInfo): TWebRequestIndy;
    function NewResponse(IndyRequest: TWebRequestIndy; Response: TIdHTTPResponseInfo): TWebResponseIndy;
    procedure CommandGet(AThread: TIdPeerThread; RequestInfo: TIdHTTPRequestInfo;
      ResponseInfo: TIdHTTPResponseInfo);
  protected
    property Server: TIdHTTPServer  read FServer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Run; override;
  end;

  TIndyFactory = class(TObject)
  private
    FApplication: TIndyApplication;
  protected
    function NewRequest(Request: TIdHTTPRequestInfo): TWebRequestIndy; virtual;
    function NewResponse(IndyRequest: TWebRequestIndy; Response: TIdHTTPResponseInfo): TWebResponseIndy; virtual;
  public
    constructor Create;
  end;

function Server: TIdHTTPServer;

implementation

uses
  SysUtils, BrkrConst, ActiveX;


function Server: TIdHTTPServer;
begin
  Result := nil;
  if Assigned(Application) then
    Result := TIndyApplication(Application).Server;
end;

{ TIndyApplication }

procedure TIndyApplication.CommandGet(AThread: TIdPeerThread;
  RequestInfo: TIdHTTPRequestInfo; ResponseInfo: TIdHTTPResponseInfo);
var
  Request: TWebRequestIndy;
  Response: TWebResponseIndy;
begin
  CoInitialize(nil);
  Response := nil;
  Request := NewRequest(RequestInfo);
  try
    Response := NewResponse(Request, ResponseInfo);
    HandleRequest(Request, Response);
  finally
    Response.Free;
    Request.Free;
  end;
  CoUnInitialize;
end;

constructor TIndyApplication.Create(AOwner: TComponent);
begin
  inherited;
  FServer := TIdHTTPServer.Create(Self);
  FServer.OnCommandGet := CommandGet;
end;

destructor TIndyApplication.Destroy;
begin
  inherited;
  FFactory.Free;
end;

function TIndyApplication.GetFactory: TIndyFactory;
begin
  if not Assigned(FFactory) then
    FFactory := TIndyFactory.Create;
  Result := FFactory;
end;

function TIndyApplication.NewRequest(Request: TIdHTTPRequestInfo): TWebRequestIndy;
begin
  Result := GetFactory.NewRequest(Request);
end;

function TIndyApplication.NewResponse(IndyRequest: TWebRequestIndy; Response: TIdHTTPResponseInfo): TWebResponseIndy;
begin
  Result := GetFactory.NewResponse(IndyRequest, Response);
end;

procedure TIndyApplication.Run;
begin
  inherited Run;
  FServer.Active := True;
end;

procedure TIndyApplication.SetFactory(Value: TIndyFactory);
begin
  Assert(FFactory = nil, 'Duplicate IndyFactory');
  FFactory := Value;
end;

{ TIndyFactory }

constructor TIndyFactory.Create;
begin
  inherited;
  FApplication := Application as TIndyApplication;
  FApplication.SetFactory(Self);
end;

function TIndyFactory.NewRequest(Request: TIdHTTPRequestInfo): TWebRequestIndy;
begin
  Result := TWebRequestIndy.Create(Request);
end;

function TIndyFactory.NewResponse(IndyRequest: TWebRequestIndy; Response: TIdHTTPResponseInfo): TWebResponseIndy;
begin
  Result := TWebResponseIndy.Create(IndyRequest, Response);
end;

procedure InitApplication;
begin
  Application := TIndyApplication.Create(nil);
end;

procedure DoneApplication;
begin
  Application.Free;
  Application := nil;
end;

initialization
  InitApplication;

finalization
  DoneApplication;

end.
