{*******************************************************}
{                                                       }
{       Indy standalone SOAP server example             }
{                                                       }
{  Pure Software Technology                             }
{                                                       }
{  Author: Dave Nottage                                 }
{                                                       }
{  Provided as-is, no warranty implied etc blah blah    }
{  Use at your own risk.                                }
{                                                       }
{  Note that not all of TWebRequestIndy and             }
{  TWebResponseIndy has been implemented, eg:           }
{  Cookies are not generated/handled                    }
{                                                       }
{                                                       }
{*******************************************************}

unit IndyHTTP;

interface

uses
  Classes, HTTPApp, IdHTTPServer;

type
  TWebRequestIndy = class(TWebRequest)
  private
    FIdRequest: TIdHTTPRequestInfo;
  protected
    { Abstract methods overridden }
    function GetStringVariable(Index: Integer): string; override;
    function GetDateVariable(Index: Integer): TDateTime; override;
    function GetIntegerVariable(Index: Integer): Integer; override;
  public
    constructor Create(IdRequest: TIdHTTPRequestInfo);
    destructor Destroy; override;
    { Abstract methods overridden }
    // Read count bytes from client
    function ReadClient(var Buffer; Count: Integer): Integer; override;
    // Read count characters as a string from client
    function ReadString(Count: Integer): string; override;
    // Translate a relative URI to a local absolute path
    function TranslateURI(const URI: string): string; override;
    // Write count bytes back to client
    function WriteClient(var Buffer; Count: Integer): Integer; override;
    // Write string contents back to client
    function WriteString(const AString: string): Boolean; override;
    // Write HTTP header string
    function WriteHeaders(StatusCode: Integer; const ReasonString, Headers: string): Boolean; override;
    function GetFieldByName(const Name: string): string; override;
    property IdRequestInfo: TIdHTTPRequestInfo  read FIdRequest;
  end;

  TWebResponseIndy = class(TWebResponse)
  private
    FIdResponse: TIdHTTPResponseInfo;
    FSent: boolean;
  protected
    { Abstract methods overridden }
    function GetStringVariable(Index: Integer): string; override;
    procedure SetStringVariable(Index: Integer; const Value: string); override;
    function GetDateVariable(Index: Integer): TDateTime; override;
    procedure SetDateVariable(Index: Integer; const Value: TDateTime); override;
    function GetIntegerVariable(Index: Integer): Integer; override;
    procedure SetIntegerVariable(Index: Integer; Value: Integer); override;
    function GetContent: string; override;
    procedure SetContent(const Value: string); override;
    function GetStatusCode: Integer; override;
    procedure SetStatusCode(Value: Integer); override;
    function GetLogMessage: string; override;
    procedure SetLogMessage(const Value: string); override;
  public
    { Abstract methods overridden }
    procedure SendResponse; override;
    procedure SendRedirect(const URI: string); override;
    procedure SendStream(AStream: TStream); override;
  public
    constructor Create(Request: TWebRequest; Response: TIdHTTPResponseInfo);
    function Sent: Boolean; override;
    procedure SetCookieField(Values: TStrings; const ADomain, APath: string;
      AExpires: TDateTime; ASecure: Boolean);
    property IdResponse: TIdHTTPResponseInfo  read FIdResponse;
  end;

implementation

uses
  SysUtils;

const
  ServerVariables: array[0..28] of string = (
    '',
    'SERVER_PROTOCOL',
    'URL',
    '',
    '',
    '',
    'HTTP_CACHE_CONTROL',
    'HTTP_DATE',
    'HTTP_ACCEPT',
    'HTTP_FROM',
    'HTTP_HOST',
    'HTTP_IF_MODIFIED_SINCE',
    'HTTP_REFERER',
    'HTTP_USER_AGENT',
    'HTTP_CONTENT_ENCODING',
    'CONTENT_TYPE',
    'CONTENT_LENGTH',
    'HTTP_CONTENT_VERSION',
    'HTTP_DERIVED_FROM',
    'HTTP_EXPIRES',
    'HTTP_TITLE',
    'REMOTE_ADDR',
    'REMOTE_HOST',
    'SCRIPT_NAME',
    'SERVER_PORT',
    '',
    'HTTP_CONNECTION',
    'HTTP_COOKIE',
    'HTTP_AUTHORIZATION');


{ TWebRequestIndy }

constructor TWebRequestIndy.Create(IdRequest: TIdHTTPRequestInfo);
begin
  FIdRequest := IdRequest;
  inherited Create;
end;

destructor TWebRequestIndy.Destroy;
begin
  //
  inherited;
end;

function TWebRequestIndy.GetDateVariable(Index: Integer): TDateTime;
begin
  Result := 0;
end;

function TWebRequestIndy.GetFieldByName(const Name: string): string;
begin
  Result := FIdRequest.Params.Values[Name];
end;

function TWebRequestIndy.GetIntegerVariable(Index: Integer): Integer;
begin
  Result := 0;
end;

{
    property Method: string index 0 read GetStringVariable;
    property Query: string index 3 read GetStringVariable;
    property PathInfo: string index 4 read GetStringVariable;
    property PathTranslated: string index 5 read GetStringVariable;
}

function TWebRequestIndy.GetStringVariable(Index: Integer): string;
begin
  case Index of
    0: Result := ExtractFileName(FIdRequest.Command); // ECB.lpszMethod;
    3: Result := ''; // ECB.lpszQueryString;
    4: Result := FIdRequest.Document; // ECB.lpszPathInfo;
    5: Result := ''; // ECB.lpszPathTranslated;
    10: Result := FIdRequest.Host;
    1..2, 6..9, 11..24, 26..28: Result := GetFieldByName(ServerVariables[Index]);
    25: Result := FIdRequest.UnparsedParams;
   else
      Result := '';
  end;
end;

function TWebRequestIndy.ReadClient(var Buffer; Count: Integer): Integer;
begin
  Result := 0;
end;

function TWebRequestIndy.ReadString(Count: Integer): string;
begin
  Result := '';
end;

function TWebRequestIndy.TranslateURI(const URI: string): string;
begin
  Result := '';
end;

function TWebRequestIndy.WriteClient(var Buffer; Count: Integer): Integer;
begin
  Result := 0;
end;

function TWebRequestIndy.WriteHeaders(StatusCode: Integer; const ReasonString, Headers: string): Boolean;
begin
  Result := True;
end;

function TWebRequestIndy.WriteString(const AString: string): Boolean;
begin
  Result := False;
end;

{ TWebResponseIndy }

constructor TWebResponseIndy.Create(Request: TWebRequest; Response: TIdHTTPResponseInfo);
begin
  inherited Create(Request);
  FIdResponse := Response;
end;

function TWebResponseIndy.GetContent: string;
begin
  Result := '';
  if Assigned(FIdResponse) then
    Result := FIdResponse.ContentText;
end;

function TWebResponseIndy.GetDateVariable(Index: Integer): TDateTime;
begin
  Result := 0;
end;

function TWebResponseIndy.GetIntegerVariable(Index: Integer): Integer;
begin
  Result := 0;
end;

function TWebResponseIndy.GetLogMessage: string;
begin
  Result := '';
end;

function TWebResponseIndy.GetStatusCode: Integer;
begin
  Result := 0;
end;

{
    property Version: string index 0 read GetStringVariable write SetStringVariable;
    property ReasonString: string index 1 read GetStringVariable write SetStringVariable;
    property Server: string index 2 read GetStringVariable write SetStringVariable;
    property WWWAuthenticate: string index 3 read GetStringVariable write SetStringVariable;
    property Realm: string index 4 read GetStringVariable write SetStringVariable;
    property Allow: string index 5 read GetStringVariable write SetStringVariable;
    property Location: string index 6 read GetStringVariable write SetStringVariable;
    property ContentEncoding: string index 7 read GetStringVariable write SetStringVariable;
    property ContentType: string index 8 read GetStringVariable write SetStringVariable;
    property ContentVersion: string index 9 read GetStringVariable write SetStringVariable;
    property DerivedFrom: string index 10 read GetStringVariable write SetStringVariable;
    property Title: string index 11 read GetStringVariable write SetStringVariable;
}

function TWebResponseIndy.GetStringVariable(Index: Integer): string;
begin
  Result := '';
  if Assigned(FIdResponse) then
    case Index of
      0, 1, 2, 4, 6, 7, 9: Result := '';
      3: Result := FIdResponse.Headers.Values['WWW-Authenticate'];
      5: Result := FIdResponse.Headers.Values['Allow'];
      8: Result := FIdResponse.ContentType;
      10: Result := FIdResponse.Headers.Values['Derived-From'];
      11: Result := FIdResponse.Headers.Values['Title'];
    end;
end;

procedure TWebResponseIndy.SendRedirect(const URI: string);
begin
  if Assigned(FIdResponse) then
  begin
    FIdResponse.Redirect(URI);
    FSent := True;
  end;
end;

procedure TWebResponseIndy.SendResponse;
begin
  FSent := True;
end;

procedure TWebResponseIndy.SendStream(AStream: TStream);
begin
  inherited;
  //
end;

function TWebResponseIndy.Sent: Boolean;
begin
  Result := FSent;
end;

procedure TWebResponseIndy.SetContent(const Value: string);
begin
  if Assigned(FIdResponse) then
    FIdResponse.ContentText := Value;
end;

procedure TWebResponseIndy.SetCookieField(Values: TStrings; const ADomain,
  APath: string; AExpires: TDateTime; ASecure: Boolean);
begin
  //
end;

procedure TWebResponseIndy.SetDateVariable(Index: Integer;
  const Value: TDateTime);
begin
  //
end;

procedure TWebResponseIndy.SetIntegerVariable(Index, Value: Integer);
begin
  //
end;

procedure TWebResponseIndy.SetLogMessage(const Value: string);
begin
  //
end;

procedure TWebResponseIndy.SetStatusCode(Value: Integer);
begin
  //
end;

procedure TWebResponseIndy.SetStringVariable(Index: Integer;
  const Value: string);
begin
  if Assigned(FIdResponse) then
    case Index of
      0, 1, 2, 4, 6, 7, 9: ;
      3: FIdResponse.Headers.Values['WWW-Authenticate'] := Value;
      5: FIdResponse.Headers.Values['Allow'] := Value;
      8: FIdResponse.ContentType := Value;
      10: FIdResponse.Headers.Values['Derived-From'] := Value;
      11: FIdResponse.Headers.Values['Title'] := Value;
    end;
end;

end.
